package de.jottyfan.timetrack.modules.done;

import java.io.Serializable;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.Map;

import de.jottyfan.timetrack.db.done.tables.records.TBillingRecord;
import de.jottyfan.timetrack.db.done.tables.records.TDoneRecord;
import de.jottyfan.timetrack.db.done.tables.records.TJobRecord;
import de.jottyfan.timetrack.db.done.tables.records.TModuleRecord;
import de.jottyfan.timetrack.db.done.tables.records.TProjectRecord;
import de.jottyfan.timetrack.modules.Bean;

/**
 * 
 * @author henkej
 *
 */
public class DoneBean implements Bean, Serializable, Comparable<DoneBean> {
	private static final long serialVersionUID = 1L;

	private static final DateTimeFormatter hhmm = DateTimeFormatter.ofPattern("HH:mm");

	private Integer pk;
	private LocalDateTime timeFrom;
	private LocalDateTime timeUntil;
	private TProjectRecord project;
	private TModuleRecord module;
	private TJobRecord activity;
	private TBillingRecord billing;

	public DoneBean() {
	}

	public DoneBean(TDoneRecord r, Map<Integer, TProjectRecord> projectMap, Map<Integer, TModuleRecord> moduleMap,
			Map<Integer, TJobRecord> jobMap, Map<Integer, TBillingRecord> billingMap) {
		this.pk = r.getPk();
		this.timeFrom = r.getTimeFrom();
		this.timeUntil = r.getTimeUntil();
		this.project = projectMap.get(r.getFkProject());
		this.module = moduleMap.get(r.getFkModule());
		this.activity = jobMap.get(r.getFkJob());
		this.billing = billingMap.get(r.getFkBilling());
	}

	/**
	 * set the day of timeFrom and timeUntil, keeping the times
	 * 
	 * @param day the day
	 */
	public void setDay(Date day) {
		if (timeFrom != null) {
			LocalDateTime ldt = timeFrom;
			LocalDate date = day.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
			ldt = ldt.withYear(date.getYear()).withMonth(date.getMonthValue()).withDayOfMonth(date.getDayOfMonth());
			timeFrom = ldt;
		}
		if (timeUntil != null) {
			LocalDateTime ldt = timeUntil;
			LocalDate date = day.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
			ldt = ldt.withYear(date.getYear()).withMonth(date.getMonthValue()).withDayOfMonth(date.getDayOfMonth());
			timeUntil = ldt;
		}
	}

	public String getTimeSummary() {
		StringBuilder buf = new StringBuilder();
		if (timeFrom != null) {
			buf.append(timeFrom.format(hhmm));
		}
		if (timeUntil != null) {
			buf.append(" - ");
			buf.append(timeUntil.format(hhmm));
		}
		return buf.toString();
	}

	@Override
	public int compareTo(DoneBean o) {
		return o == null || timeFrom == null || o.getTimeFrom() == null ? 0 : timeFrom.compareTo(o.getTimeFrom());
	}

	public String getTimeDiff() {
		LocalDateTime earlier = timeFrom != null ? timeFrom : LocalDateTime.now();
		LocalDateTime later = timeUntil != null ? timeUntil : LocalDateTime.now();
		Duration diff = Duration.between(earlier, later);
		return String.format("%02d:%02d", diff.toHours(), diff.toMinutes() % 60);
	}

	/**
	 * get local date time from s
	 * 
	 * @param s   the HH:mm formatted values
	 * @param ldt the date as basic for that datetime, for today one can use
	 *            LocalDateTime.now(); in fact this is set if ldt is null internally
	 * @return the generated datetime
	 */
	public LocalDateTime getLocalDateTimeFromHHmm(String s, LocalDateTime ldt) {
		if (s == null || s.trim().isEmpty()) {
			return null;
		} else if (ldt == null) {
			ldt = LocalDateTime.now();
		}
		String[] hm = s.split(":");
		Integer hours = 0;
		Integer minutes = 0;
		if (hm.length > 0) {
			hours = Integer.valueOf(hm[0]);
		}
		if (hm.length > 1) {
			minutes = Integer.valueOf(hm[1]);
		}
		ldt = ldt.withHour(hours);
		ldt = ldt.withMinute(minutes);
		ldt = ldt.withSecond(0);
		ldt = ldt.withNano(0);
		return ldt;
	}

	public String getProjectName() {
		return project == null ? "" : project.getName();
	}

	public String getModuleName() {
		return module == null ? "" : module.getName();
	}

	public String getJobName() {
		return activity == null ? "" : activity.getName();
	}
	
	public String getBillingName() {
		return billing == null ? "" : billing.getName();
	}
	
	public String getBillingShortcut() {
		return billing == null ? "" : billing.getShortcut();
	}
	
	public String getBillingCsskey() {
		return billing == null ? "" : billing.getCsskey();
	}

	public String getTimeFromString() {
		return timeFrom == null ? "" : timeFrom.format(hhmm);
	}

	public void setTimeFromString(String s) {
		this.timeFrom = getLocalDateTimeFromHHmm(s, null); // use setDay instead
	}

	public String getTimeUntilString() {
		return timeUntil == null ? "" : timeUntil.format(hhmm);
	}

	public void setTimeUntilString(String s) {
		this.timeUntil = getLocalDateTimeFromHHmm(s, null); // use setDay instead
	}

	public Integer getPk() {
		return this.pk;
	}

	public void setPk(Integer pk) {
		this.pk = pk;
	}

	public LocalDateTime getTimeFrom() {
		return timeFrom;
	}

	public void setTimeFrom(LocalDateTime timeFrom) {
		this.timeFrom = timeFrom;
	}

	public LocalDateTime getTimeUntil() {
		return timeUntil;
	}

	public void setTimeUntil(LocalDateTime timeUntil) {
		this.timeUntil = timeUntil;
	}

	public TProjectRecord getProject() {
		return project;
	}

	public void setProject(TProjectRecord project) {
		this.project = project;
	}

	public TModuleRecord getModule() {
		return module;
	}

	public void setModule(TModuleRecord module) {
		this.module = module;
	}

	public TJobRecord getActivity() {
		return activity;
	}

	public void setActivity(TJobRecord activity) {
		this.activity = activity;
	}

	/**
	 * @return the billing
	 */
	public TBillingRecord getBilling() {
		return billing;
	}

	/**
	 * @param billing the billing to set
	 */
	public void setBilling(TBillingRecord billing) {
		this.billing = billing;
	}
}
